# coding: utf-8

"""
    Managed Ray API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from openapi_client.configuration import Configuration


class ExecuteCommandResponse(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'command_id': 'str',
        'directory_name': 'str',
        'dns_address': 'str'
    }

    attribute_map = {
        'command_id': 'command_id',
        'directory_name': 'directory_name',
        'dns_address': 'dns_address'
    }

    def __init__(self, command_id=None, directory_name=None, dns_address=None, local_vars_configuration=None):  # noqa: E501
        """ExecuteCommandResponse - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._command_id = None
        self._directory_name = None
        self._dns_address = None
        self.discriminator = None

        self.command_id = command_id
        self.directory_name = directory_name
        self.dns_address = dns_address

    @property
    def command_id(self):
        """Gets the command_id of this ExecuteCommandResponse.  # noqa: E501


        :return: The command_id of this ExecuteCommandResponse.  # noqa: E501
        :rtype: str
        """
        return self._command_id

    @command_id.setter
    def command_id(self, command_id):
        """Sets the command_id of this ExecuteCommandResponse.


        :param command_id: The command_id of this ExecuteCommandResponse.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and command_id is None:  # noqa: E501
            raise ValueError("Invalid value for `command_id`, must not be `None`")  # noqa: E501

        self._command_id = command_id

    @property
    def directory_name(self):
        """Gets the directory_name of this ExecuteCommandResponse.  # noqa: E501


        :return: The directory_name of this ExecuteCommandResponse.  # noqa: E501
        :rtype: str
        """
        return self._directory_name

    @directory_name.setter
    def directory_name(self, directory_name):
        """Sets the directory_name of this ExecuteCommandResponse.


        :param directory_name: The directory_name of this ExecuteCommandResponse.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and directory_name is None:  # noqa: E501
            raise ValueError("Invalid value for `directory_name`, must not be `None`")  # noqa: E501

        self._directory_name = directory_name

    @property
    def dns_address(self):
        """Gets the dns_address of this ExecuteCommandResponse.  # noqa: E501


        :return: The dns_address of this ExecuteCommandResponse.  # noqa: E501
        :rtype: str
        """
        return self._dns_address

    @dns_address.setter
    def dns_address(self, dns_address):
        """Sets the dns_address of this ExecuteCommandResponse.


        :param dns_address: The dns_address of this ExecuteCommandResponse.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and dns_address is None:  # noqa: E501
            raise ValueError("Invalid value for `dns_address`, must not be `None`")  # noqa: E501

        self._dns_address = dns_address

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ExecuteCommandResponse):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, ExecuteCommandResponse):
            return True

        return self.to_dict() != other.to_dict()
